@@ -1,4 +1,4 @@
-/* $EPIC: irc.c,v 1.705 2004/02/21 17:12:18 keerf Exp $ */
+/* $EPIC: irc.c,v 1.742 2006/03/04 16:29:43 jnelson Exp $ */
 /*
  * ircII: a new irc client.  I like it.  I hope you will too!
  *
@@ -40,19 +40,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC4-2.0";
-const char useful_info[] = "epic4 2 0 0";
+const char irc_version[] = "EPIC4-2.4";
+const char useful_info[] = "epic4 2 4 0";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20040219";
+const char internal_version[] = "20060303";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 710;
+const unsigned long	commit_id = 745;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -60,7 +60,7 @@ const unsigned long	commit_id = 710;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Circumlocution";
+const char ridiculous_version_name[] = "Languish";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -764,7 +764,7 @@ static	void	parse_args (int argc, char **argv)
 		exit(1);
 	}
 
-	if (!check_nickname(nickname, 1))
+	if (strcmp(nickname, "0") && !check_nickname(nickname, 1))
 	{
 		fprintf(stderr, "Invalid nickname: [%s]\n", nickname);
 		fprintf(stderr, "Please restart IRC II with a valid nickname\n");
@@ -963,7 +963,7 @@ static void check_password (void)
 #define INPUT_PASSWD_LEN 15
 	char 	input_passwd[INPUT_PASSWD_LEN];
 #ifdef HAVE_GETPASS
-	strlcpy(input_passwd, getpass("Passwd: "), sizeof input_password);
+	strlcpy(input_passwd, getpass("Passwd: "), sizeof input_passwd);
 #else
 	fprintf(stderr, "Passwd: ");
 	fgets(input_passwd, INPUT_PASSWD_LEN - 1, stdin);
