@@ -1,4 +1,4 @@
-/* $EPIC: keys.c,v 1.30 2003/10/28 05:53:57 jnelson Exp $ */
+/* $EPIC: keys.c,v 1.32 2005/04/20 04:07:02 wd Exp $ */
 /*
  * keys.c:  Keeps track of what happens whe you press a key.
  *
@@ -101,25 +101,42 @@ struct Binding *add_binding (const char *name, BindFunction func, char *alias) {
     return bp;
 }
 
+
+static void remove_bound_keys(struct Key *, struct Binding *);
 void remove_binding (char *name) {
     struct Binding *bp;
 
     if (!name)
-	return;
+        return;
 
     bp = (struct Binding *)remove_from_list((List **)&binding_list, name);
     if (bp) {
-	new_free(&bp->name);
-	if (bp->alias)
-	    new_free(&bp->alias);
-	if (bp->filename)
-	    new_free(&bp->filename);
-	new_free(&bp);
+        /* be sure to remove any keys bound to this binding first */
+        remove_bound_keys(head_keymap, bp);
+
+        new_free(&bp->name);
+        if (bp->alias)
+            new_free(&bp->alias);
+        if (bp->filename)
+            new_free(&bp->filename);
+        new_free(&bp);
     }
 
     return;
 }
 
+static void remove_bound_keys (struct Key *map, struct Binding *binding) {
+    int c;
+
+    for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
+        if (map[c].bound == binding)
+            map[c].bound = NULL;
+        if (map[c].map)
+            remove_bound_keys(map[c].map, binding);
+    }
+}
+
+
 struct Binding *find_binding (const char *name) {
     if (!name)
 	return NULL;
@@ -365,10 +382,10 @@ struct Key *timeout_keypress (struct Key *last, Timeval pressed) {
 }
 
 struct Key *construct_keymap (struct Key *owner) {
-    unsigned char c;
+    int c;
     struct Key *map = new_malloc(sizeof(struct Key) * KEYMAP_SIZE);
 
-    for (c = 0;c < KEYMAP_SIZE - 1;c++) {
+    for (c = 0;c <= KEYMAP_SIZE - 1;c++) {
 	map[c].val = c;
 	map[c].bound = NULL;
 	map[c].map = NULL;
@@ -387,7 +404,7 @@ struct Key *construct_keymap (struct Key *owner) {
  * are not left around in the timeout system.  the function returns positive
  * if *the map passed* was removed, negative otherwise. */
 int clean_keymap (struct Key *map) {
-    unsigned char c;
+    int c;
     int save = 0;
 
     /* walk through the map to see if things are in use.  if something is
@@ -396,7 +413,7 @@ int clean_keymap (struct Key *map) {
      * the map is saved, 0 otherwise.  we walk through all keys here, even
      * if we know we're going to save the map early on.  this allows the
      * cleaner to catch dead submaps in the current map. */
-    for (c = 0; c < KEYMAP_SIZE - 1;c++) {
+    for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
 	if (map[c].bound)
 	    save = 1; /* key in use.  save map. */
 	if (map[c].map) {
@@ -633,14 +650,14 @@ struct Key *find_sequence (const unsigned char *seq, int slen) {
 /* init_keys:  initialize default keybindings that apply without terminal
  * specificity.  we use the above functions to take care of this */
 void init_keys (void) {
-    unsigned char c;
+    int c;
     unsigned char s[2];
     head_keymap = construct_keymap(NULL);
 
 #define BIND(x, y) bind_string(x, y, NULL);
     /* bind characters 32 - 255 to SELF_INSERT. */
     s[1] = '\0';
-    for (c = 32;c < KEYMAP_SIZE - 1;c++) {
+    for (c = 32;c <= KEYMAP_SIZE - 1;c++) {
 	s[0] = c;
 	BIND(s, "SELF_INSERT");
     }
@@ -777,7 +794,7 @@ void save_bindings (FILE *fp, int do_all) {
 }
 
 void save_bindings_recurse (FILE *fp, struct Key *map, const unsigned char *str, size_t len) {
-    unsigned char c;
+    int c;
     unsigned char *newstr;
     unsigned char *ds; /* decompressed sequence */
     size_t size;
@@ -790,7 +807,7 @@ void save_bindings_recurse (FILE *fp, struct Key *map, const unsigned char *str,
     /* go through our map, see what is changed, and save it.  recurse down
      * as necessary. */
     newstr[len + 1] = '\0';
-    for (c = 0; c < KEYMAP_SIZE - 1;c++) {
+    for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
 	newstr[len] = c;
 	if (map[c].bound && map[c].changed) {
 	    bind_string_decompress(ds, newstr, len + 1);
@@ -815,11 +832,11 @@ void remove_bindings (void) {
 }
 
 void remove_bindings_recurse (struct Key *map) {
-    unsigned char c;
+    int c;
 
     /* go through our map, clear any memory that might be left lying around.
      * recurse as necessary */
-    for (c = 0; c < KEYMAP_SIZE - 1;c++) {
+    for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
 	if (map[c].map)
 	    remove_bindings_recurse(map[c].map);
 	if (map[c].stuff)
@@ -851,10 +868,10 @@ void unload_bindings (const char *pkg) {
 }
 
 void unload_bindings_recurse (const char *pkg, struct Key *map) {
-    unsigned char c;
+    int c;
 
     /* go through, see which keys are package specific, unload them. */
-    for (c = 0; c < KEYMAP_SIZE - 1;c++) {
+    for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
 	/* if the key is explicitly bound to something, and it was done in
 	 * our package, unbind it. */
 	if (map[c].bound && !my_stricmp(map[c].filename, pkg)) {
@@ -1085,7 +1102,7 @@ doc/keys distributed with the EPIC source."
  * keybindings.  given a map and a string to work from.  if the string is
  * NULL, the function recurses through the entire map. */
 void show_all_bindings (struct Key *map, const unsigned char *str, size_t len) {
-    unsigned char c;
+    int c;
     unsigned char *newstr;
     struct Binding *self_insert;
     size_t size;
@@ -1097,7 +1114,7 @@ void show_all_bindings (struct Key *map, const unsigned char *str, size_t len) {
 
     /* show everything in our map.  recurse down. */
     newstr[len + 1] = '\0';
-    for (c = 0; c < KEYMAP_SIZE - 1;c++) {
+    for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
 	newstr[len] = c;
 	if (map[c].map || (map[c].bound && map[c].bound != self_insert))
 	    show_key(&map[c], newstr, len + 1, 1);
@@ -1154,7 +1171,7 @@ BUILT_IN_COMMAND(rbindcmd) {
 }
 
 void show_all_rbindings (struct Key *map, const unsigned char *str, int len, struct Binding *binding) {
-    unsigned char c;
+    int c;
     unsigned char *newstr;
     size_t size;
 
@@ -1165,7 +1182,7 @@ void show_all_rbindings (struct Key *map, const unsigned char *str, int len, str
     /* this time, show only those things bound to our function, and call on
      * ourselves to recurse instead. */
     newstr[len + 1] = '\0';
-    for (c = 0; c < KEYMAP_SIZE - 1;c++) {
+    for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
 	newstr[len] = c;
 	if (map[c].bound == binding)
 	    show_key(&map[c], newstr, len + 1, 0);
@@ -1375,7 +1392,7 @@ char *bindctl (char *input)
 }
 
 void bindctl_getmap (struct Key *map, const unsigned char *str, int len, char **ret) {
-    unsigned char c;
+    int c;
     unsigned char *newstr;
     unsigned char *decomp;
     size_t size;
@@ -1387,7 +1404,7 @@ void bindctl_getmap (struct Key *map, const unsigned char *str, int len, char **
 
     /* grab all keys that are bound, put them in ret, and continue. */
     newstr[len + 1] = '\0';
-    for (c = 1; c < KEYMAP_SIZE - 1;c++) {
+    for (c = 1; c <= KEYMAP_SIZE - 1;c++) {
 	newstr[len] = c;
 	if (map[c].bound)
 	    malloc_strcat_wordlist(ret, " ", bind_string_decompress(decomp, newstr, len + 1));
