@@ -1,4 +1,4 @@
-/* $EPIC: server.c,v 1.109 2004/01/24 15:00:53 crazyed Exp $ */
+/* $EPIC: server.c,v 1.113 2005/06/04 04:32:53 jnelson Exp $ */
 /*
  * server.c:  Things dealing with that wacky program we call ircd.
  *
@@ -140,6 +140,7 @@ void 	add_to_server_list (const char *server, int port, const char *password, co
 		s->nickname = (char *) 0;
 		s->s_nickname = (char *) 0;
 		s->d_nickname = (char *) 0;
+		s->unique_id = NULL;
 		s->userhost = (char *) 0;
 		s->registered = 0;
 		s->eof = 0;
@@ -281,6 +282,7 @@ static 	void 	remove_from_server_list (int i, int override)
 	new_free(&s->ison_queue);		/* XXX Aren't these free? */
 	new_free(&s->who_queue);
 	new_free(&s->invite_channel);
+	new_free(&s->unique_id);
 	new_free(&s->last_notify_nick);
 	new_free(&s->joined_nick);
 	new_free(&s->public_nick);
@@ -2170,7 +2172,8 @@ int	get_server_port (int refnum)
 	if (!(s = get_server(refnum)))
 		return 0;
 
-	if (!inet_ntostr((SA *)&s->remote_sockname, NULL, 0, p_port, 12, 0))
+	if (is_server_open(refnum))
+	    if (!inet_ntostr((SA *)&s->remote_sockname, NULL, 0, p_port, 12, 0))
 		return atol(p_port);
 
 	return s->port;
@@ -2184,7 +2187,8 @@ int	get_server_local_port (int refnum)
 	if (!(s = get_server(refnum)))
 		return 0;
 
-	if (!inet_ntostr((SA *)&s->remote_sockname, NULL, 0, p_port, 12, 0))
+	if (is_server_open(refnum))
+	    if (!inet_ntostr((SA *)&s->local_sockname, NULL, 0, p_port, 12, 0))
 		return atol(p_port);
 
 	return 0;
@@ -2311,6 +2315,7 @@ void	change_server_nickname (int refnum, const char *nick)
 {
 	Server *s;
 	char *	n;
+	const char *id;
 
 	if (!(s = get_server(refnum)))
 		return;			/* Uh, no. */
@@ -2318,14 +2323,20 @@ void	change_server_nickname (int refnum, const char *nick)
 	s->resetting_nickname = 0;
 	if (nick)
 	{
-		n = LOCAL_COPY(nick);
-		if ((n = check_nickname(n, 1)) != NULL)
-		{
-		    malloc_strcpy(&s->d_nickname, n);
-		    malloc_strcpy(&s->s_nickname, n);
-		}
-		else
+	    /* If changing to our Unique ID, the default nickname is 0 */
+	    id = get_server_unique_id(refnum);
+	    if (id && !my_stricmp(nick, id))
+		malloc_strcpy(&s->d_nickname, zero);
+	    else
+	    {
+	        n = LOCAL_COPY(nick);
+                if (!(n = check_nickname(n, 1)))
 			reset_nickname(refnum);
+		else
+			malloc_strcpy(&s->d_nickname, n);
+	    }
+
+	    malloc_strcpy(&s->s_nickname, n);
 	}
 
 	if (s->s_nickname)
@@ -2345,15 +2356,23 @@ const char *	get_pending_nickname (int refnum)
 void	accept_server_nickname (int refnum, const char *nick)
 {
 	Server *s;
+	const char *id;
 
 	if (!(s = get_server(refnum)))
 		return;
 
+	/* We always accept whatever the server says our new nick is */
 	malloc_strcpy(&s->nickname, nick);
-	malloc_strcpy(&s->d_nickname, nick);
 	new_free(&s->s_nickname);
 	s->fudge_factor = 0;
 
+	/* Change our default nickname to our new nick, or 0 for unique id's */
+	id = get_server_unique_id(refnum);
+	if (id && !my_stricmp(nick, id))
+		malloc_strcpy(&s->d_nickname, zero);
+	else
+		malloc_strcpy(&s->d_nickname, nick);
+
 	if (refnum == primary_server)
 		strlcpy(nickname, nick, sizeof nickname);
 
@@ -2638,6 +2657,21 @@ SACCESSOR(message, quit_message, "get_server_quit_message")
 SACCESSOR(cookie, cookie, NULL)
 SACCESSOR(ver, version_string, NULL)
 
+GET_SATTRIBUTE(unique_id, NULL)
+void    set_server_unique_id (int servref, const char * id)
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return;
+
+	malloc_strcpy(&s->unique_id , id);
+	if (id && s->d_nickname && !my_stricmp(id, s->d_nickname))
+		malloc_strcpy(&s->d_nickname, zero);
+}
+
+
+
 GET_IATTRIBUTE(operator)
 void	set_server_operator (int refnum, int flag)
 {
@@ -3008,6 +3042,9 @@ char 	*serverctl 	(char *input)
 		} else if (!my_strnicmp(listc, "PORT", len)) {
 			num = get_server_port(refnum);
 			RETURN_INT(num);
+		} else if (!my_strnicmp(listc, "LOCALPORT", len)) {
+			num = get_server_local_port(refnum);
+			RETURN_INT(num);
 		} else if (!my_strnicmp(listc, "QUIT_MESSAGE", len)) {
 			ret = get_server_quit_message(refnum);
 			RETURN_STR(ret);
@@ -3020,6 +3057,9 @@ char 	*serverctl 	(char *input)
 		} else if (!my_strnicmp(listc, "UMODES", len)) {
 			ret = get_possible_umodes(refnum);
 			RETURN_STR(ret);
+		} else if (!my_strnicmp(listc, "UNIQUE_ID", len)) {
+			ret = get_server_unique_id(refnum);
+			RETURN_STR(ret);
 		} else if (!my_strnicmp(listc, "USERHOST", len)) {
 			ret = get_server_userhost(refnum);
 			RETURN_STR(ret);
@@ -3096,6 +3136,8 @@ char 	*serverctl 	(char *input)
 			RETURN_EMPTY;		/* Read only for now */
 		} else if (!my_strnicmp(listc, "UMODES", len)) {
 			set_possible_umodes (refnum, input);
+		} else if (!my_strnicmp(listc, "UNIQUE_ID", len)) {
+			set_server_unique_id(refnum, input);
 		} else if (!my_strnicmp(listc, "USERHOST", len)) {
 			set_server_userhost(refnum, input);
 		} else if (!my_strnicmp(listc, "VERSION", len)) {
